@@ -1,3 +1,8 @@
+//! Reporting
+//!
+//! This goes more into the direction of instrumentation. Unfortunately
+//! `tokio` uses the word `Instrumentation` already for their tracing
+//! implementation.
 use std::time::Duration;
 
 use crate::InclusiveRange;
@@ -7,9 +12,19 @@ pub use simple_reporter::*;
 pub trait ReporterFactory: Send + Sync + 'static {
     type ReporterType: Reporter;
 
+    /// Create a new [Reporter].
+    ///
+    /// This might share state with the factory or not
     fn make(&self) -> Self::ReporterType;
 }
 
+/// A Reporter is an interface to track occurences of different kinds
+///
+/// Implementors can use this to put instrumentation on downloads
+/// or simply to log.
+///
+/// All methods should return quickly to not to influence the
+/// downloading too much with measuring.
 #[allow(unused_variables)]
 pub trait Reporter: Clone + Send + Sync + 'static {
     /// The effective range which is downloaded (and split into parts)
@@ -18,8 +33,12 @@ pub trait Reporter: Clone + Send + Sync + 'static {
     fn download_started(&self) {}
     /// IO tasks finished
     ///
-    /// **This always is the last method called on a [Reporter].**
-    fn download_finished(&self) {}
+    /// **This always is the last method called on a [Reporter] if the download was successful.**
+    fn download_completed(&self) {}
+    /// IO tasks finished
+    ///
+    /// **This always is the last method called on a [Reporter] if the download failed.**
+    fn download_failed(&self) {}
     /// All queues are full so no new request could be scheduled
     fn queue_full(&self) {}
     /// A part was completed
@@ -30,6 +49,7 @@ pub trait Reporter: Clone + Send + Sync + 'static {
     fn part_completed(&self, part_index: usize, n_chunks: usize, n_bytes: usize, time: Duration) {}
 }
 
+/// Disables reporting
 #[derive(Copy, Clone)]
 pub struct NoReporting;
 
@@ -43,9 +63,10 @@ impl ReporterFactory for NoReporting {
 }
 
 mod simple_reporter {
+    //! Simple reporting with counters
     use std::{
         sync::{
-            atomic::{AtomicU64, AtomicUsize, Ordering},
+            atomic::{AtomicBool, AtomicU64, AtomicUsize, Ordering},
             Arc, Mutex,
         },
         time::{Duration, Instant},
@@ -53,6 +74,7 @@ mod simple_reporter {
 
     use super::{Reporter, ReporterFactory};
 
+    /// Creates [SimpleReporter]s
     pub struct SimpleReporterFactory;
     impl ReporterFactory for SimpleReporterFactory {
         type ReporterType = SimpleReporter;
@@ -62,6 +84,7 @@ mod simple_reporter {
         }
     }
 
+    /// A `SimpleReporter` collects metrics and creates a [SimpleReport]
     #[derive(Clone)]
     pub struct SimpleReporter {
         inner: Arc<Inner>,
@@ -82,6 +105,10 @@ mod simple_reporter {
             self.inner.download_finished_at.lock().unwrap().is_some()
         }
 
+        /// Get a [SimpleReport].
+        ///
+        /// Takes a snapshot. A download might still be running when a snapshot
+        /// is taken.
         pub fn report(&self) -> SimpleReport {
             let inner = self.inner.as_ref();
             let download_time =
@@ -99,20 +126,21 @@ mod simple_reporter {
 
             SimpleReport {
                 is_finished: self.is_download_finished(),
+                is_failed: inner.is_failed.load(Ordering::SeqCst),
                 download_time,
                 bytes_per_second: bytes_per_second_f64 as u64,
-                megabytes_per_second: (bytes_per_second_f64 / 1_000_000.0) as u64,
-                mebibytes_per_second: (bytes_per_second_f64 / 1_048_576.0) as u64,
-                gigabits_per_second: (bytes_per_second_f64 / 1_000_000_000.0) as u64 * 8,
-                gibibits_per_second: ((bytes_per_second_f64 * 8.0) / 1_073_741_824.0) as u64,
+                megabytes_per_second: bytes_per_second_f64 / 1_000_000.0,
+                mebibytes_per_second: bytes_per_second_f64 / 1_048_576.0,
+                gigabits_per_second: (bytes_per_second_f64 * 8.0) / 1_000_000_000.0,
+                gibibits_per_second: (bytes_per_second_f64 * 8.0) / 1_073_741_824.0,
                 n_queue_full: inner.n_queue_full.load(Ordering::SeqCst),
                 n_bytes_received,
                 n_chunks_received: inner.n_chunks_received.load(Ordering::SeqCst),
                 n_parts_received: inner.n_parts_received.load(Ordering::SeqCst),
                 min_chunk_bytes: inner.min_chunk_bytes.load(Ordering::SeqCst),
                 max_chunk_bytes: inner.max_chunk_bytes.load(Ordering::SeqCst),
-                min_chunk_time: Duration::from_millis(inner.min_chunk_ms.load(Ordering::SeqCst)),
-                max_chunk_time: Duration::from_millis(inner.max_chunk_ms.load(Ordering::SeqCst)),
+                min_chunk_time: Duration::from_micros(inner.min_chunk_us.load(Ordering::SeqCst)),
+                max_chunk_time: Duration::from_micros(inner.max_chunk_us.load(Ordering::SeqCst)),
                 min_part_bytes: inner.min_part_bytes.load(Ordering::SeqCst),
                 max_part_bytes: inner.max_part_bytes.load(Ordering::SeqCst),
                 min_chunks_per_part: inner.min_chunks_per_part.load(Ordering::SeqCst),
@@ -133,14 +161,15 @@ mod simple_reporter {
     pub struct SimpleReport {
         /// `true` if the download was finished
         pub is_finished: bool,
+        pub is_failed: bool,
         /// If the download is not yet finished, this is the time
         /// elapsed since the start of the download.
         pub download_time: Duration,
         pub bytes_per_second: u64,
-        pub megabytes_per_second: u64,
-        pub mebibytes_per_second: u64,
-        pub gigabits_per_second: u64,
-        pub gibibits_per_second: u64,
+        pub megabytes_per_second: f64,
+        pub mebibytes_per_second: f64,
+        pub gigabits_per_second: f64,
+        pub gibibits_per_second: f64,
         pub n_queue_full: usize,
         pub n_bytes_received: usize,
         pub n_chunks_received: usize,
@@ -162,10 +191,15 @@ mod simple_reporter {
             *self.inner.download_started_at.lock().unwrap() = Instant::now();
         }
 
-        fn download_finished(&self) {
+        fn download_completed(&self) {
             *self.inner.download_finished_at.lock().unwrap() = Some(Instant::now());
         }
 
+        fn download_failed(&self) {
+            *self.inner.download_finished_at.lock().unwrap() = Some(Instant::now());
+            self.inner.is_failed.store(true, Ordering::SeqCst);
+        }
+
         fn queue_full(&self) {
             self.inner.n_queue_full.fetch_add(1, Ordering::SeqCst);
         }
@@ -175,9 +209,9 @@ mod simple_reporter {
             inner.n_chunks_received.fetch_add(1, Ordering::SeqCst);
             inner.min_chunk_bytes.fetch_min(n_bytes, Ordering::SeqCst);
             inner.max_chunk_bytes.fetch_max(n_bytes, Ordering::SeqCst);
-            let ms = time.as_millis() as u64;
-            inner.min_chunk_ms.fetch_min(ms, Ordering::SeqCst);
-            inner.max_chunk_ms.fetch_max(ms, Ordering::SeqCst);
+            let us = time.as_micros() as u64;
+            inner.min_chunk_us.fetch_min(us, Ordering::SeqCst);
+            inner.max_chunk_us.fetch_max(us, Ordering::SeqCst);
         }
 
         fn part_completed(
@@ -207,14 +241,15 @@ mod simple_reporter {
     struct Inner {
         download_started_at: Mutex<Instant>,
         download_finished_at: Mutex<Option<Instant>>,
+        is_failed: AtomicBool,
         n_queue_full: AtomicUsize,
         n_bytes_received: AtomicUsize,
         n_chunks_received: AtomicUsize,
         n_parts_received: AtomicUsize,
         min_chunk_bytes: AtomicUsize,
         max_chunk_bytes: AtomicUsize,
-        min_chunk_ms: AtomicU64,
-        max_chunk_ms: AtomicU64,
+        min_chunk_us: AtomicU64,
+        max_chunk_us: AtomicU64,
         min_part_bytes: AtomicUsize,
         max_part_bytes: AtomicUsize,
         min_chunks_per_part: AtomicUsize,
@@ -228,14 +263,15 @@ mod simple_reporter {
             Inner {
                 download_started_at: Mutex::new(Instant::now()),
                 download_finished_at: Mutex::new(None),
+                is_failed: AtomicBool::new(false),
                 n_bytes_received: AtomicUsize::new(0),
                 n_chunks_received: AtomicUsize::new(0),
                 n_parts_received: AtomicUsize::new(0),
                 n_queue_full: AtomicUsize::new(0),
                 min_chunk_bytes: AtomicUsize::new(usize::MAX),
                 max_chunk_bytes: AtomicUsize::new(0),
-                min_chunk_ms: AtomicU64::new(u64::MAX),
-                max_chunk_ms: AtomicU64::new(0),
+                min_chunk_us: AtomicU64::new(u64::MAX),
+                max_chunk_us: AtomicU64::new(0),
                 min_part_bytes: AtomicUsize::new(usize::MAX),
                 max_part_bytes: AtomicUsize::new(0),
                 min_chunks_per_part: AtomicUsize::new(usize::MAX),
